// debug header
// indicates whether debugging/error message should be displayed
//
// Author:
//   2001/2002 Pierre Lombard <pierre.lombard@imag.fr>
//
// License: GPLv2

#ifndef __DBG_H__
#define __DBG_H__ 1

#ifndef SHOWERR
# define SHOWERR 1
#endif

#define SHOWINFO 1
#ifndef SHOWINFO
# define SHOWINFO 0
#endif

#if SHOWERR || SHOWINFO
 #include <sys/types.h>
 #include <unistd.h>
 #include <errno.h>
 #include <stdio.h>
 #include <string.h>
 #include <stdarg.h>
#endif

#if !MSWIN
#define DISPLAY_STDERR_MACRO(infotype,format,args...) do {		\
	int sav = errno;						\
	const int ERRBUFSIZE = 100;					\
	char errbuf[ERRBUFSIZE + 1];					\
	fprintf(stderr, "%s (%s) pid=%d : ",				\
			(infotype), __FILE__, getpid());		\
	fprintf(stderr, format "\n" ,## args);				\
	if (strerror_r(sav, errbuf, ERRBUFSIZE)) {			\
		fprintf(stderr, "[maybe meaningless: errno=%d [%s]]\n",	\
			       	sav, errbuf);				\
	} else {							\
		fprintf(stderr, "Error in error: cannot "		\
				"strerror_r() errno=%d\n", errno);	\
	}								\
	errno = sav;							\
} while (0)

#define DISPLAY_DEVNULL_MACRO(infotype,format,args...) do { } while (0)
static inline void DISPLAY_STDERR_FUNC(const char * const strinfo, const char * const format, ...) {
	int sav = errno;
#define __DISPLAY_STDERR_FUNC_ERRBUFSIZE 100
	char errbuf[__DISPLAY_STDERR_FUNC_ERRBUFSIZE+1];
	va_list ap;

	fprintf(stderr, "%s (%s) pid=%d : ",
			strinfo, __FILE__, getpid());
	va_start(ap, format);
	vfprintf(stderr, format, ap);
	va_end(ap);
	if (strerror_r(sav, errbuf, __DISPLAY_STDERR_FUNC_ERRBUFSIZE)) {
		fprintf(stderr, "[maybe meaningless: errno=%d [%s]]\n",
			sav, errbuf);
	} else {
		fprintf(stderr, "Error in error: cannot "
			"strerror_r() errno=%d\n", errno);
	}
        errno = sav;
}


#if SHOWERR
 #define ERR(format,args...) DISPLAY_STDERR_MACRO("ERROR", format, ##args)
#else
 #define ERR(format,args...) DISPLAY_DEVNULL_MACRO("ERROR", format, ##args)
#endif

#if SHOWINFO
 #define INFO(format,args...) DISPLAY_STDERR_MACRO("INFO", format, ##args)
#else
 #define INFO(format,args...) DISPLAY_DEVNULL_MACRO("INFO", format, ##args)
#endif
#endif


#define DISP_WARN "WARNING"
#define DISP_INFO "INFO"
#define DISP_ERR "ERROR"
#define DISP_FATAL "ERROR"


static inline void DISP(const char * const strinfo, const char * const format, ...) {
	int sav = errno;
	int ret;
#define __DISP_LOGBUFSIZE 256
	char logbuf[__DISP_LOGBUFSIZE + 1];
	int written = 0;
	char *ptr = logbuf;
	va_list ap;

	ret = snprintf(ptr + written, __DISP_LOGBUFSIZE - written,
			"%s (%s) pid=%d : ",
			strinfo, __FILE__, getpid());
	if (ret == -1) {
		goto do_log;
	}
	written += ret;

	va_start(ap, format);
	ret = vsnprintf(ptr + written, __DISP_LOGBUFSIZE - written, format, ap);
	va_end(ap);
	if (ret == -1) {
		goto do_log;
	}
	written += ret;

#ifdef HAVE_STRERROR_R
	{
#define __DISP_ERRBUFSIZE 100
		const int __DISP_ERRBUFSIZE = 100;
		char errbuf[__DISP_ERRBUFSIZE + 1];
		if (strerror_r(sav, errbuf, __DISP_ERRBUFSIZE)) {
			ret = snprintf(ptr + written, __DISP_LOGBUFSIZE - written,
					" [errno=%d \"%s\"] ",
					sav, errbuf);
		} else {
			ret = snprintf(ptr + written, __DISP_LOGBUFSIZE - written,
					"\nDOUBLE-Error: cannot strerror_r() errno=%d\n",
				        errno);
		}
	}
#else
	{
		char *errmsg = strerror(sav);
		if (errmsg) {
			ret = snprintf(ptr + written, __DISP_LOGBUFSIZE - written,
					" [errno=%d \"%s\"]\n",
					sav, errmsg);
		} else {
			ret = snprintf(ptr + written, __DISP_LOGBUFSIZE - written,
					"\nDOUBLE-Error: cannot strerror() errno=%d\n",
				        errno);
		}
	}
#endif
	if (ret == -1) {
		goto do_log;
	}
	written += ret;

do_log:
	logbuf[__DISP_LOGBUFSIZE] = '\0';

//#warning "plug the log system here"
	fprintf(stderr, "%s\n", logbuf);

        errno = sav;
}



//static inline ERR(const char * const format, ...) {

//}




// Macro used to display numerical IPs
// Reentrant as it does not use static buffers (like in inet_ntoa())
// Example: printf("IP of i=%d is " NTODSTR "\n", i, NTOD(x));
#define NTODSTR "%d.%d.%d.%d:%d"
#define NTOD(x) ((x).sin_addr.s_addr>>0)&0xFF, \
		((x).sin_addr.s_addr>>8)&0xFF, \
		((x).sin_addr.s_addr>>16)&0xFF, \
		((x).sin_addr.s_addr>>24)&0xFF, \
		ntohs((x).sin_port)

#endif

