#ifndef VIODS_SUPPORT

#include "nfsp_iods.h"

#include "default.h"

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>
#include <netinet/in.h>
#include <netdb.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <errno.h>

#include "logging.h"
#include "ipstring.h"



/* Number of iods */
int iods_nb = 0;

/* Network parameters of iods */
struct sockaddr_in iods[MAX_IODS];

// file is the configuration file to parsed
// 
// This function parses the configuration file and fills the data on iods: iods[] and iods_nb.
// It also initialize the socket used to communicate with iods (UDP): iods_udp_fd
// 
// A configuration file matches the following pattern : host:port[ ]*[#comment]\n
// NB: order matters, no blank line, no comment line
// XXX this primitive parser should be cleanly rewritten
void init_iods(const char * const file) {		// used in nfsd.c
	const int MAXLINELEN = 512;
	FILE *f;
	char buf[MAXLINELEN + 1];
	int errors = 0;
	int line_no = 0;
	int iod_no = 0;
	int i;

	Dprintf(D_NFSP_IODS, "init_iods(): start");

	
	for (i = 0; i < MAX_IODS; ++i) {
		memset(&(iods[i]), '\0', sizeof(iods[i]));
	}

	Dprintf(D_NFSP_IODS, "init_iods(): parsing the config file: %s",file);
	f = fopen(file, "r");
	if (NULL == f) {
		Dprintf(L_FATAL, "could not open %s errno=%d",file,errno);
		exit(1);
	}

	while (NULL != fgets(buf,MAXLINELEN,f)) {
		char *host = NULL;
		char *port = NULL;

		line_no++;
		if (line_no > MAX_IODS) {
			Dprintf(L_FATAL, "too many iod servers in %s (MAX_IODS=%d)",
				file, MAX_IODS);
			exit(1);
		}

		// Cleanups and split of the host/port
		do {
			char *foo, *bar;

			// If a # is found the line is over
			if (NULL != (foo = strchr(buf,'#'))) {
				*foo = '\0';
			}

			// Remove any space left in buf
			foo = buf;
			bar = buf;
			while(*foo) {
				if (isblank(*foo)) {
					foo++;
				} else {
					*bar++ = *foo++;
				}
			}
			*bar = '\0';

			// Skip the blank lines
			if (0 == strlen(buf)) {
				Dprintf(D_NFSP_IODS, "blank line found: skipping it");
				continue;
			}

			// buf now hopefully contain "host:port" so it is now splitted into
			// host & port on the ':'
			if (NULL == (foo = strchr(buf,':')) ) {
				errors++;
				Dprintf(L_WARNING, "line %d of %s: MUST be host:port", line_no, file);
				continue;
			}
			host = buf;
			*foo = '\0';
			port = foo+1;
		} while (0);
		Dprintf(D_NFSP_IODS, "iod %d %s : %s", iod_no, host, port);

		// fill the sockaddr struct
		iods[iod_no].sin_family = AF_INET;
		do {
			struct hostent *he;
			he = gethostbyname(host);
			if (NULL != he) {
				memmove((char*)&(iods[iod_no].sin_addr),
					he->h_addr, he->h_length);
			} else {
				Dprintf(L_WARNING, "could not resolve host=%s errno=%d", host, errno);
				errors++;
			}
		} while (0);
		do {
			long tmp;
			tmp = strtol(port, (char**) NULL, 10);
			if (tmp<1 || tmp>65535) {
				errors++;
				Dprintf(L_WARNING, "invalid port must be in 1:65535 range (was read as %ld)", tmp);
			}
			iods[iod_no].sin_port = htons(tmp);
		}while (0);
		iod_no++;

	} // process next line

	if (errors) {
		Dprintf(L_FATAL, "found %d errors in file %s",errors,file);
		exit(1);
	}
	iods_nb = iod_no;

	Dprintf(D_NFSP_IODS, "found %d iod's :", iods_nb);
	for(i = 0; i < iods_nb; ++i) {
		Dprintf(D_NFSP_IODS, "iod=%d is " NTODSTR , i, NTOD(iods[i]));
	}

	if (0 == iods_nb) {
		Dprintf(L_FATAL, "FATAL: there must be at least 1 iod");
		exit(1);
	}

	if (EOF == fclose(f)) {
		Dprintf(L_WARNING, "WARNING: fclose() failed (errno=%d)", errno);
	}

	Dprintf(D_NFSP_IODS, "init_iods() end");
}

#endif /* !VIODS_SUPPORT */

