// Author: Pierre Lombard <pierre.lombard@imag.fr>
// Date: 2001/2002
// License: GPLv2

#include <unistd.h>
#include <fcntl.h>
#include <errno.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <sys/poll.h>
#include <stdio.h>
#include <stdlib.h>

#include "timed_connect.h"

// timed_connect() has the same behavior as connect() but you can specify a
// timeout_ms in milliseconds
//
// Return values: -1   error 0    success
//
// errno: not set so it's the last system call that sets it but you do not know
// exactly which one

#define verbose_printf(x...) do { int tmp = errno; printf(##x); errno = tmp; } while (0)
//#define verbose_printf(x...) do { } while (0)

int timed_connect(int s, const struct sockaddr *serv_addr, socklen_t addrlen, unsigned long timeout_ms) {
	int ret;
	int retval = -1;
	int flags;

	if (!timeout_ms) {
		return connect(s, serv_addr, addrlen);
	}


	ret = fcntl(s, F_GETFL);
	if (ret == -1) {
		verbose_printf("fcntl() 1 errno=%d\n", errno);
		return -1;
	}
	flags = ret;

	ret = fcntl(s, F_SETFL, flags | O_NONBLOCK);
	if (ret == -1) {
		verbose_printf("fcntl() 2 errno=%d\n", errno);
		return -1;
	}

	ret = connect(s, serv_addr, addrlen);
	if (ret == -1) {
		if (errno == EINPROGRESS) {
			struct pollfd pollset;

			pollset.fd = s;
			pollset.events = POLLOUT;
			pollset.revents = 0;

			ret = poll(&pollset, 1, timeout_ms);
			switch (ret) {
				case -1:
					verbose_printf("poll() failed: errno=%d", errno);
					break;
				case 0:
					verbose_printf("timeout_msed");
					errno =	ETIMEDOUT;
					goto out;
					break;
				case 1:
					if (pollset.revents & POLLOUT) {
						int opt;
						int opt_len = sizeof(opt);
						
						verbose_printf("connect should be ready\n");

						ret = getsockopt(s, SOL_SOCKET, SO_ERROR, &opt, &opt_len);
						if (ret == -1) {
							verbose_printf("getsockopt() errno=%d\n", errno);
							return -1;
						}
						if (opt == 0) {
							verbose_printf("excellent connect() suceeded\n");
							retval = 0;
						} else {
							verbose_printf("too bad connect() failed: errno=%d", opt);
							errno = opt;
						}

					} else {
						verbose_printf("other flags ignored\n");
					}
			}
		} else {
			verbose_printf("connect() errno=%d\n", errno);
		}
	
	}

out:
	ret = fcntl(s, F_SETFL, flags);
	if (ret == -1) {
		verbose_printf("fcntl() 3 errno=%d\n", errno);
		retval = -1;
	}

	return retval;
}
